<?php

namespace Drupal\lupus_ce_renderer\EventSubscriber;

use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Cache\CacheableRedirectResponse;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Render\BubbleableMetadata;
use Drupal\lupus_ce_renderer\Cache\CustomElementsJsonResponse;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\HttpKernel\Event\FilterResponseEvent;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Handles redirects and provides them as explicit API responses.
 */
class CustomElementsRedirectResponseSubscriber implements EventSubscriberInterface {

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Constructs the object.
   *
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   *   The module handler.
   */
  public function __construct(ModuleHandlerInterface $moduleHandler) {
    $this->moduleHandler = $moduleHandler;
  }

  /**
   * Handles the response.
   *
   * @param \Symfony\Component\HttpKernel\Event\FilterResponseEvent $event
   *   The event to process.
   */
  public function onResponse(FilterResponseEvent $event) {
    $result = $event->getResponse();
    $request = $event->getRequest();

    // When custom elements rendering is requested, handle the redirect and
    // provide them as explicit API responses. So the client may forward the
    // request to the browser.
    if ($request->getRequestFormat() == 'custom_elements' && $result instanceof RedirectResponse) {
      $data = [];
      $url = $result->getTargetUrl();
      // Translate absolute external URLs to relative URLs.
      if (UrlHelper::isExternal($url) && UrlHelper::externalIsLocal($url, $GLOBALS['base_url'])) {
        $parsed_url = parse_url($url);
        $url = $parsed_url['path'] ?? '/';
        $url .= isset($parsed_url['query']) ? '?' . $parsed_url['query'] : '';
        $url .= isset($parsed_url['fragment']) ? '#' . $parsed_url['fragment'] : '';
      }

      $data['redirect'] = [
        'external' => UrlHelper::isExternal($url),
        'url' => $url,
        'statusCode' => $result->getStatusCode(),
      ];
      $bubbleable_metadata = new BubbleableMetadata();
      if ($result instanceof CacheableRedirectResponse) {
        $bubbleable_metadata->addCacheableDependency($result->getCacheableMetadata());
      }
      $this->moduleHandler->alter('lupus_ce_renderer_response', $data, $bubbleable_metadata, $request);
      $response = new CustomElementsJsonResponse();
      $response->setData($data);
      $response->addCacheableDependency($bubbleable_metadata);
      $event->setResponse($response);
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events = [];
    $events[KernelEvents::RESPONSE][] = ['onResponse', 0];
    return $events;
  }

}
